\name{GoSteep-method}
\docType{methods}
\alias{GoSteep}
\title{Steepest descent in the space of measures.}

\description{
  Main subroutine implementing the gradient descent optimisation in the space of measures.
}

\usage{ GoSteep(object, number.of.steps, ...)
}

\arguments{
  \item{object}{An object of class \code{MesOp}.}
  \item{number.of.steps}{Number of steps to be performed in the steepest descent direction.}
  \item{...}{further arguments of
    \code{object.function} and \code{gradient.function}. See example
  section below.}
}

\value{Returns object of class \code{MesOp}. %% with the following slots
    %% \itemize{
    %%   \item space: finite grid on which \code{measure} is defined.
    %%   \item measure: vector over which \code{object.function} is optimised.
    %%   \item object.function: function to be optimised, must
    %%   have a signature \code{value <- object.function(measure, space, ...)}.
    %%   \item gradient.function: gradient of the \code{object.function},
    %%   must have a signature \code{gradient.vector <- gradient.function(measure, space, ...)}.
    %%   \item minim: if \bold{TRUE} the function is minimised and it is maximised otherwise.
    %%   \item smallest.step.size: gradient descent terminates if the
    %%   current step size is smaller than \code{smallest.step.size}.
    %%   %%\item step.algo: algorithm for choosing the step size, 0 - line
    %%   %% search algorithm, 1 - Armijo algorithm.
    %%   \item step.size: size of the step to be performed in the direction
    %%   of the gradient/anti-gradient.
    %%   \item tolerance.measure: zero-value threshold parameter for
    %%   discarding atoms of a negligible size.
    %%   \item tolerance.gradient: threshold parameter to decide when the
    %%   coordinates of the gradient vector are sufficiently small.
    %%   \item uni: if \bold{TRUE} then we spread the mass uniformly among the
    %%   points where the gradient is minimal.
    %%   \item details: if \bold{TRUE} prints extra information
    %%   on progress.
    %%   % \item visualise:
    %% }
}

\details{The function starts by calculating the gradient vector for the
  current measure. In the main loop a
  \code{step.size} for the next iteration is selected according to the line search
  algorithm.%% if \code{step.algo=0}. Otherwise \code{step.size} is chosen
  %% according to the Armijo algorithm.
  After that a vector \code{incremental.measure} with
  \code{sum(abs(incremental.measure)) == step.size} is selected in such a way
  that it provides a steepest descent direction at the point
  \code{measure}. It minimises linear form \code{gradient.function(measure, space, ...) *
  incremental.measure} over all vectors with total absolute sum less then
  \code{step.size} under constraint that \code{measure +
  incremental.measure} is vector with positive coordinates.
  The main loop terminates if either current gradient vector satisfies the
  optimal conditions, i.e. it \code{all(grad >= -tolerance.gradient)
           && all(grad[measure >= tolerance.measure] <=
  tolerance.gradient)}, or \code{step.size < smallest.step.size}, or
  prescribed \code{number.of.steps} has been performed. 

  The realisation follows the algorithmic implementation
  presented in Sec. 5 of [1].}

\examples{
## Application of steepest descent method
## to a CoF with k = 1 and ChF functionals.
## See [1] for details.
\dontrun{
library(mesop)
## Bortkiewicz horse kick data gives the number
## of Prussian soldiers killed by a horse kick
## in 10 cavalry corps over a 20 years period.
## See L. Bortkiewicz. (1898) Das gesetz der kleinen
## zahlen. Monatshefte fuer Mathematik und Physic., 9(1).
sample <- c(rep(0, 109), rep(1, 65), rep(2, 22), rep(3, 3), rep(4, 1))
## Grid for measure.
grid.m <- seq(0, 10, by=1)
## Grid for empirical distribution function.
grid.x <- seq(-2, 5, 0.1)
## Empirical distribution function for the sample.
emp.cdf <- ecdf(sample)(grid.x)
## Pairwise sums of sample elements
## for convolution of empirical distribution function.
## combn generates all different pairs (i_{1}, i_{2}) of indices inside
## 1:length(sample), then we calculate sample[i_{1}] + sample[i_{2}].
sample.conv <- apply(combn(length(sample), 2), 2, function(col){
    return(sample[col[1]] + sample[col[2]])})
## Calculating the estimate of convolution of empirical distribution
## function with itself, according to Frees.
## See E.W. Frees.p (1986) Nonparametric renewal function
##  estimation. Ann. Statist., 14(4):1366--1378.
emp.cdf.conv <- ecdf(sample.conv)(grid.x)
## Precalculations of empirical distribution function in the points x-v,
## where x belongs to grid.x  and v belongs to grid.m, see above.
emp.cdf.shift <- matrix(unlist(lapply(grid.m, FUN = function(v, grid.x, sample){
    ecdf(sample)(grid.x - v)}, grid.x, sample)),
    nrow = length(grid.x), ncol = length(grid.m))

## Starting measure for a gradient descent method.
measure <- rep(20 / length(grid.m), length(grid.m))
measure[grid.m == 0] <- 0

ObjectFunction <- function(measure, grid.m, emp.cdf, emp.cdf.conv, emp.cdf.shift){
  return(sum((emp.cdf.conv - emp.cdf - (emp.cdf.shift - emp.cdf) \%*\% measure)**2))
}

GradientFunction <- function(measure, grid.m, emp.cdf, emp.cdf.conv, emp.cdf.shift){
  return((-2) * t(emp.cdf.conv - emp.cdf - (emp.cdf.shift - emp.cdf) \%*\% measure)
  \%*\% (emp.cdf.shift - emp.cdf))
}


## Grid for empirical characteristic function.
grid.cf <- seq(-5, 5, by=0.1)

IntegralOfCosine <- function(t, measure, grid.m){
  if(length(grid.m) != (length(measure))) browser()
  return(sum((cos(t * grid.m) - 1) * measure))
}

IntegralOfSine <-  function(t, measure, grid.m){
  if(length(grid.m) != (length(measure))) browser()
  return(sum(sin(t * grid.m) * measure))
}

## Values of the real part of empirical characteristic function
ecf.re <- sapply(grid.cf, function(t){ (1 / length(sample)) * sum(cos(t * sample))})
## Values of the imaginary part of empirical characteristic function
ecf.im <- sapply(grid.cf, function(t){ (1 / length(sample)) * sum(sin(t * sample))})

ObjectFunctionCF <- function(measure, grid.m, grid.cf, ecf.re, ecf.im){
    re <- sapply(grid.cf, function(t){ exp(IntegralOfCosine(t, measure, grid.m)) *
    cos(IntegralOfSine(t, measure, grid.m))})
    im <- sapply(grid.cf, function(t){ exp(IntegralOfCosine(t, measure, grid.m)) *
    sin(IntegralOfSine(t, measure, grid.m))})
    return(sum((re - ecf.re)^2 + (im - ecf.im)^2))                     
}

GradientFunctionCF <- function(measure, grid.m, grid.cf, ecf.re, ecf.im){
    return(sapply(grid.m, function(x){2 *
    sum((sapply(grid.cf, function(t){exp(IntegralOfCosine(t, measure, grid.m)) *
    cos(IntegralOfSine(t, measure, grid.m))}) - ecf.re) *
    sapply(grid.cf, function(t){exp(IntegralOfCosine(t, measure, grid.m)) *
    (cos(IntegralOfSine(t, measure, grid.m)) * (cos(t * x) - 1)  -
    sin(IntegralOfSine(t, measure, grid.m)) * sin(t * x))}) +
    (sapply(grid.cf, function(t){exp(IntegralOfCosine(t, measure, grid.m)) *
    sin(IntegralOfSine(t, measure, grid.m))}) - ecf.im) *
    sapply(grid.cf, function(t){exp(IntegralOfCosine(t, measure, grid.m)) *
    (sin(IntegralOfSine(t, measure, grid.m)) * (cos(t * x) - 1) +
    cos(IntegralOfSine(t, measure, grid.m)) * sin(t * x))}))}))
}

## Initialisation of a new object and making one step in the direction of steepest descent.
opt1 <- new("MesOp", space=grid.m, measure=measure,
object.function=ObjectFunction, gradient.function=GradientFunction,
emp.cdf=emp.cdf, emp.cdf.conv=emp.cdf.conv, emp.cdf.shift=emp.cdf.shift)
## Making 500 steps in the steepest descent direction.
opt1 <- GoSteep(opt1, 500)

## Initialisation of a new object by measure opt1@measure and
## making one step in the direction of steepest descent.
opt.cf <- new("MesOp", space=grid.m, measure=opt1@measure,
object.function=ObjectFunctionCF, gradient.function=GradientFunctionCF,
step.algo=0, grid.cf=grid.cf, ecf.re=ecf.re, ecf.im=ecf.im,
tolerance.measure=10^{-3}, tolerance.gradient=10^{-3})

## Making 500 steps in the steepest descent direction.
opt.cf <- GoSteep(opt.cf, 500)

## Plotting the gradien and the estimated measure.
plot(opt.cf)
## the gradient function is 0 on the support of the measure and non-negative elsewhere,
## so the necessary conditions for a minimum are fulfilled.
## Alternative plot of the estimated measure:
mes <- opt.cf@measure
mes.max <- max(mes[grid.m >= 0])
mes.points <- mes[grid.m >= 0]
mes.points[mes.points == 0] <- NA
plot(grid.m[grid.m >= 0], mes.points, main="", xlab="x", ylab="",
    ylim=c(0, mes.max + 0.01 * mes.max), pch=3)
abline(v=0:11, col="lightgrey", lty=2)
abline(h=seq(0, mes.max + 0.01 * mes.max, by=0.1), col="lightgrey", lty=2)
points(grid.m[grid.m >= 0], mes.points, pch=3)
legend("topright", cex=1.3, legend=c(expression(paste("||",
    tilde(Lambda)[1], "|| = ", 0.6109))), bg="white")
legend("top", cex=1.3, legend=c(expression(tilde(Lambda)[1])), pch=3, bg="White")

## Poisson distribution corresponds to a single atom concenrated at 1.
## Compare its mass: mes.points[2]=0.5979 with the sample mean of the sample: mean(sample)=0.61.
## Both are the estimated parameter of the Poisson distribution.
## So if we ignore small atoms at positions 2 and 4 totaling to 0.013
## we can say that Poisson distribution is confirmed.
}
}

\references{ [1] A. Lindo, S. Zuyev and S. Sagitov. (2015) Nonparametric estimation of infinitely divisible
  distributions based on variational analysis on measures.
  arXiv:1510.04968.}
  

\author{Alexey Lindo, email: \email{lindo@chalmers.se} \cr
  Sergei Zuyev, email: \email{sergei.zuyev@chalmers.se}.}

\seealso{\code{\link[mesop]{mesop}}, \code{\link{MesOp}},
    \code{\link{summary}} and \code{\link{plot}}. 
  }

\keyword{optimization}